
package de.codedo.conflicteditor;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.hasSize;
import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.List;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;
import org.junit.Ignore;
import org.junit.Test;

public class DocumentMatcherTestCase extends Object
{
    private ObjectMapper _mapper = new ObjectMapper();

    @Test
    public void equalDocumentsShouldProduceNoDifferences() throws Exception
    {
        JsonNode node = parse("{ 'key' : 'value' }");
        List<Difference> diffs = new DocumentMatcher(node, node).compare();
        assertThat(diffs, hasSize(0));
    }

    @Test
    public void differentValuesShouldProduceOneDifferencePerKey() throws Exception
    {
        JsonNode original = parse("{ 'key' : 'value' }");
        JsonNode conflict = parse("{ 'key' : 'other-value' }");
        List<Difference> diffs = new DocumentMatcher(original, conflict).compare();
        assertEquals(1, diffs.size());

        Difference diff = diffs.get(0);
        assertEquals("key", diff.key);
        assertEquals("value", diff.currentValue);
        assertEquals("other-value", diff.otherValue);
    }

    @Ignore("does not work yet")
    @Test
    public void valuePresentOnlyInOriginalShouldProduceDifference() throws Exception
    {
        JsonNode original = parse("{ 'key' : 'value', 'only-here' : 'test' }");
        JsonNode conflict = parse("{ 'key' : 'value' }");
        List<Difference> diffs = new DocumentMatcher(original, conflict).compare();
        assertThat(diffs, hasSize(1));
    }

    @Ignore("not yet implemented")
    @Test
    public void valueOnlyInConflict() throws Exception
    {
    }

    private JsonNode parse(String input) throws IOException
    {
        // Jackson cannot parse JSON that uses single quotes ...
        String convertedInput = input.replace("'", "\"");
        return _mapper.readTree(convertedInput);
    }
}
